@@ -17,6 +17,8 @@
  * (C) Copyright 2010 - 2011 Red Hat, Inc.
  */
 
+#include "config.h"
+
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <dbus/dbus.h>
@@ -152,7 +154,6 @@ static gboolean find_device_for_connection (NmCli *nmc, NMConnection *connection
                                             const char *nsp, NMDevice **device, const char **spec_object, GError **error);
 static const char *active_connection_state_to_string (NMActiveConnectionState state);
 static void active_connection_state_cb (NMActiveConnection *active, GParamSpec *pspec, gpointer user_data);
-static void activate_connection_cb (NMClient *client, const char *path, GError *error, gpointer user_data);
 static void get_connections_cb (NMRemoteSettings *settings, gpointer user_data);
 static NMCResultCode do_connections_list (NmCli *nmc, int argc, char **argv);
 static NMCResultCode do_connections_status (NmCli *nmc, int argc, char **argv);
@@ -402,7 +403,7 @@ show_connection (NMConnection *data, gpointer user_data)
 	char *timestamp_str;
 	char timestamp_real_str[64];
 
-	s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+	s_con = nm_connection_get_setting_connection (connection);
 	if (s_con) {
 		/* Obtain field values */
 		timestamp = nm_setting_connection_get_timestamp (s_con);
@@ -437,7 +438,7 @@ find_connection (GSList *list, const char *filter_type, const char *filter_val)
 	iterator = list;
 	while (iterator) {
 		connection = NM_CONNECTION (iterator->data);
-		s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+		s_con = nm_connection_get_setting_connection (connection);
 		if (s_con) {
 			id = nm_setting_connection_get_id (s_con);
 			uuid = nm_setting_connection_get_uuid (s_con);
@@ -586,7 +587,7 @@ show_active_connection (gpointer data, gpointer user_data)
 
 		if (!strcmp (active_path, con_path)) {
 			/* This connection is active */
-			s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+			s_con = nm_connection_get_setting_connection (connection);
 			g_assert (s_con != NULL);
 
 			/* Obtain field values */
@@ -680,7 +681,7 @@ check_ethernet_compatible (NMDeviceEthernet *device, NMConnection *connection, G
 
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+	s_con = nm_connection_get_setting_connection (connection);
 	g_assert (s_con);
 
 	connection_type = nm_setting_connection_get_connection_type (s_con);
@@ -694,7 +695,7 @@ check_ethernet_compatible (NMDeviceEthernet *device, NMConnection *connection, G
 	if (!strcmp (connection_type, NM_SETTING_PPPOE_SETTING_NAME))
 		is_pppoe = TRUE;
 
-	s_wired = (NMSettingWired *) nm_connection_get_setting (connection, NM_TYPE_SETTING_WIRED);
+	s_wired = nm_connection_get_setting_wired (connection);
 	/* Wired setting is optional for PPPoE */
 	if (!is_pppoe && !s_wired) {
 		g_set_error (error, 0, 0,
@@ -736,7 +737,7 @@ check_wifi_compatible (NMDeviceWifi *device, NMConnection *connection, GError **
 
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+	s_con = nm_connection_get_setting_connection (connection);
 	g_assert (s_con);
 
 	if (strcmp (nm_setting_connection_get_connection_type (s_con), NM_SETTING_WIRELESS_SETTING_NAME)) {
@@ -745,7 +746,7 @@ check_wifi_compatible (NMDeviceWifi *device, NMConnection *connection, GError **
 		return FALSE;
 	}
 
-	s_wireless = NM_SETTING_WIRELESS (nm_connection_get_setting (connection, NM_TYPE_SETTING_WIRELESS));
+	s_wireless = nm_connection_get_setting_wireless (connection);
 	if (!s_wireless) {
 		g_set_error (error, 0, 0,
 		             "The connection was not a valid WiFi connection.");
@@ -794,7 +795,7 @@ check_bt_compatible (NMDeviceBt *device, NMConnection *connection, GError **erro
 
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+	s_con = nm_connection_get_setting_connection (connection);
 	g_assert (s_con);
 
 	if (strcmp (nm_setting_connection_get_connection_type (s_con), NM_SETTING_BLUETOOTH_SETTING_NAME)) {
@@ -803,7 +804,7 @@ check_bt_compatible (NMDeviceBt *device, NMConnection *connection, GError **erro
 		return FALSE;
 	}
 
-	s_bt = NM_SETTING_BLUETOOTH (nm_connection_get_setting (connection, NM_TYPE_SETTING_BLUETOOTH));
+	s_bt = nm_connection_get_setting_bluetooth (connection);
 	if (!s_bt) {
 		g_set_error (error, 0, 0,
 		             "The connection was not a valid Bluetooth connection.");
@@ -853,7 +854,7 @@ check_olpc_mesh_compatible (NMDeviceOlpcMesh *device, NMConnection *connection,
 
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+	s_con = nm_connection_get_setting_connection (connection);
 	g_assert (s_con);
 
 	if (strcmp (nm_setting_connection_get_connection_type (s_con), NM_SETTING_OLPC_MESH_SETTING_NAME)) {
@@ -862,7 +863,7 @@ check_olpc_mesh_compatible (NMDeviceOlpcMesh *device, NMConnection *connection,
 		return FALSE;
 	}
 
-	s_mesh = NM_SETTING_OLPC_MESH (nm_connection_get_setting (connection, NM_TYPE_SETTING_OLPC_MESH));
+	s_mesh = nm_connection_get_setting_olpc_mesh (connection);
 	if (!s_mesh) {
 		g_set_error (error, 0, 0,
 		             "The connection was not a valid Mesh connection.");
@@ -885,7 +886,7 @@ check_wimax_compatible (NMDeviceWimax *device, NMConnection *connection, GError
 
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+	s_con = nm_connection_get_setting_connection (connection);
 	g_assert (s_con);
 
 	if (strcmp (nm_setting_connection_get_connection_type (s_con), NM_SETTING_WIMAX_SETTING_NAME)) {
@@ -894,7 +895,7 @@ check_wimax_compatible (NMDeviceWimax *device, NMConnection *connection, GError
 		return FALSE;
 	}
 
-	s_wimax = NM_SETTING_WIMAX (nm_connection_get_setting (connection, NM_TYPE_SETTING_WIMAX));
+	s_wimax = nm_connection_get_setting_wimax (connection);
 	if (!s_wimax) {
 		g_set_error (error, 0, 0,
 		             "The connection was not a valid WiMAX connection.");
@@ -930,7 +931,7 @@ check_modem_compatible (NMDeviceModem *device, NMConnection *connection, GError
 
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+	s_con = nm_connection_get_setting_connection (connection);
 	g_assert (s_con);
 
 	/* Figure out what the modem supports */
@@ -942,7 +943,7 @@ check_modem_compatible (NMDeviceModem *device, NMConnection *connection, GError
 			return FALSE;
 		}
 
-		s_gsm = NM_SETTING_GSM (nm_connection_get_setting (connection, NM_TYPE_SETTING_GSM));
+		s_gsm = nm_connection_get_setting_gsm (connection);
 		if (!s_gsm) {
 			g_set_error (error, 0, 0,
 				     "The connection was not a valid GSM connection.");
@@ -955,7 +956,7 @@ check_modem_compatible (NMDeviceModem *device, NMConnection *connection, GError
 			return FALSE;
 		}
 
-		s_cdma = NM_SETTING_CDMA (nm_connection_get_setting (connection, NM_TYPE_SETTING_CDMA));
+		s_cdma = nm_connection_get_setting_cdma (connection);
 		if (!s_cdma) {
 			g_set_error (error, 0, 0,
 				     "The connection was not a valid CDMA connection.");
@@ -991,40 +992,6 @@ nm_device_is_connection_compatible (NMDevice *device, NMConnection *connection,
 	return FALSE;
 }
 
-
-/**
- * nm_client_get_active_connection_by_path:
- * @client: a #NMClient
- * @object_path: the object path to search for
- *
- * Gets a #NMActiveConnection from a #NMClient.
- *
- * Returns: the #NMActiveConnection for the given @object_path or %NULL if none is found.
- **/
-static NMActiveConnection *
-nm_client_get_active_connection_by_path (NMClient *client, const char *object_path)
-{
-	const GPtrArray *actives;
-	int i;
-	NMActiveConnection *active = NULL;
-
-	g_return_val_if_fail (NM_IS_CLIENT (client), NULL);
-	g_return_val_if_fail (object_path, NULL);
-
-	actives = nm_client_get_active_connections (client);
-	if (!actives)
-		return NULL;
-
-	for (i = 0; i < actives->len; i++) {
-		NMActiveConnection *candidate = g_ptr_array_index (actives, i);
-		if (!strcmp (nm_object_get_path (NM_OBJECT (candidate)), object_path)) {
-			active = candidate;
-			break;
-		}
-	}
-
-	return active;
-}
 /* -------------------- */
 
 static NMActiveConnection *
@@ -1100,7 +1067,7 @@ find_device_for_connection (NmCli *nmc,
 	g_return_val_if_fail (spec_object != NULL && *spec_object == NULL, FALSE);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-	s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+	s_con = nm_connection_get_setting_connection (connection);
 	g_assert (s_con);
 	con_type = nm_setting_connection_get_connection_type (s_con);
 
@@ -1220,6 +1187,8 @@ active_connection_state_to_string (NMActiveConnectionState state)
 		return _("activating");
 	case NM_ACTIVE_CONNECTION_STATE_ACTIVATED:
 		return _("activated");
+	case NM_ACTIVE_CONNECTION_STATE_DEACTIVATING:
+		return _("deactivating");
 	case NM_ACTIVE_CONNECTION_STATE_UNKNOWN:
 	default:
 		return _("unknown");
@@ -1349,79 +1318,32 @@ timeout_cb (gpointer user_data)
 }
 
 static void
-foo_active_connections_changed_cb (NMClient *client,
-                                   GParamSpec *pspec,
-                                   gpointer user_data)
-{
-	/* Call again activate_connection_cb with dummy arguments;
-	 * the correct ones are taken from its first call.
-	 */
-	activate_connection_cb (NULL, NULL, NULL, NULL);
-}
-
-static void
-activate_connection_cb (NMClient *client, const char *path, GError *error, gpointer user_data)
+activate_connection_cb (NMClient *client, NMActiveConnection *active, GError *error, gpointer user_data)
 {
 	NmCli *nmc = (NmCli *) user_data;
-	NMActiveConnection *active;
 	NMActiveConnectionState state;
-	static gulong handler_id = 0;
-	static NmCli *orig_nmc;
-	static const char *orig_path;
-	static GError *orig_error;
 
-	if (nmc)
-	{
-		/* Called first time; store actual arguments */
-		orig_nmc = nmc;
-		orig_path = path;
-		orig_error = error;
-	}
-
-	/* Disconnect the handler not to be run any more */
-	if (handler_id != 0) {
-		g_signal_handler_disconnect (orig_nmc->client, handler_id);
-		handler_id = 0;
-	}
-
-	if (orig_error) {
-		g_string_printf (orig_nmc->return_text, _("Error: Connection activation failed: %s"), orig_error->message);
-		orig_nmc->return_value = NMC_RESULT_ERROR_CON_ACTIVATION;
+	if (error) {
+		g_string_printf (nmc->return_text, _("Error: Connection activation failed: %s"), error->message);
+		nmc->return_value = NMC_RESULT_ERROR_CON_ACTIVATION;
 		quit ();
 	} else {
-		active = nm_client_get_active_connection_by_path (orig_nmc->client, orig_path);
-		if (!active) {
-			/* The active connection path is not in active connections list yet; wait for active-connections signal. */
-			/* This is basically the case for VPN connections. */
-			if (nmc) {
-				/* Called first time, i.e. by nm_client_activate_connection() */
-				handler_id = g_signal_connect (orig_nmc->client, "notify::active-connections",
-				                               G_CALLBACK (foo_active_connections_changed_cb), NULL);
-				return;
-			} else {
-				g_string_printf (orig_nmc->return_text, _("Error: Obtaining active connection for '%s' failed."), orig_path);
-				orig_nmc->return_value = NMC_RESULT_ERROR_CON_ACTIVATION;
-				quit ();
-				return;
-			}
-		}
-
 		state = nm_active_connection_get_state (active);
 
 		printf (_("Active connection state: %s\n"), active_connection_state_to_string (state));
-		printf (_("Active connection path: %s\n"), orig_path);
+		printf (_("Active connection path: %s\n"), nm_object_get_path (NM_OBJECT (active)));
 
-		if (orig_nmc->nowait_flag || state == NM_ACTIVE_CONNECTION_STATE_ACTIVATED) {
+		if (nmc->nowait_flag || state == NM_ACTIVE_CONNECTION_STATE_ACTIVATED) {
 			/* don't want to wait or already activated */
 			quit ();
 		} else {
 			if (NM_IS_VPN_CONNECTION (active))
-				g_signal_connect (NM_VPN_CONNECTION (active), "vpn-state-changed", G_CALLBACK (vpn_connection_state_cb), orig_nmc);
+				g_signal_connect (NM_VPN_CONNECTION (active), "vpn-state-changed", G_CALLBACK (vpn_connection_state_cb), nmc);
 			else
-				g_signal_connect (active, "notify::state", G_CALLBACK (active_connection_state_cb), orig_nmc);
+				g_signal_connect (active, "notify::state", G_CALLBACK (active_connection_state_cb), nmc);
 
 			/* Start timer not to loop forever when signals are not emitted */
-			g_timeout_add_seconds (orig_nmc->timeout, timeout_cb, orig_nmc);
+			g_timeout_add_seconds (nmc->timeout, timeout_cb, nmc);
 		}
 	}
 }
@@ -1434,7 +1356,6 @@ do_connection_up (NmCli *nmc, int argc, char **argv)
 	gboolean device_found;
 	NMConnection *connection = NULL;
 	NMSettingConnection *s_con;
-	const char *con_path;
 	const char *con_type;
 	const char *iface = NULL;
 	const char *ap = NULL;
@@ -1541,9 +1462,7 @@ do_connection_up (NmCli *nmc, int argc, char **argv)
 	/* create NMClient */
 	nmc->get_client (nmc);
 
-	con_path = nm_connection_get_path (connection);
-
-	s_con = (NMSettingConnection *) nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
+	s_con = nm_connection_get_setting_connection (connection);
 	g_assert (s_con);
 	con_type = nm_setting_connection_get_connection_type (s_con);
 
@@ -1559,12 +1478,14 @@ do_connection_up (NmCli *nmc, int argc, char **argv)
 		goto error;
 	}
 
-	/* Use nowait_flag instead of should_wait because exitting has to be postponed till active_connection_state_cb()
-	 * is called, giving NM time to check our permissions */
+	/* Use nowait_flag instead of should_wait because exiting has to be postponed till
+	 * active_connection_state_cb() is called. That gives NM time to check our permissions
+	 * and we can follow activation progress.
+	 */
 	nmc->nowait_flag = !wait;
 	nmc->should_wait = TRUE;
 	nm_client_activate_connection (nmc->client,
-	                               con_path,
+	                               connection,
 	                               device,
 	                               spec_object,
 	                               activate_connection_cb,
