@@ -482,46 +482,37 @@ nm_device_wimax_get_bsid (NMDeviceWimax *self)
 	return priv->bsid;
 }
 
-static GSList *
-filter_connections (NMDevice *device, const GSList *connections)
+static gboolean
+connection_valid (NMDevice *device, NMConnection *connection)
 {
-	GSList *filtered = NULL;
-	const GSList *iter;
+	NMSettingConnection *s_con;
+	NMSettingWimax *s_wimax;
+	const char *ctype;
+	const GByteArray *mac;
+	const char *hw_str;
+	struct ether_addr *hw_mac;
 
-	for (iter = connections; iter; iter = g_slist_next (iter)) {
-		NMConnection *candidate = NM_CONNECTION (iter->data);
-		NMSettingConnection *s_con;
-		NMSettingWimax *s_wimax;
-		const char *ctype;
-		const GByteArray *mac;
-		const char *hw_str;
-		struct ether_addr *hw_mac;
+	s_con = nm_connection_get_setting_connection (connection);
+	g_assert (s_con);
 
-		s_con = (NMSettingConnection *) nm_connection_get_setting (candidate, NM_TYPE_SETTING_CONNECTION);
-		g_assert (s_con);
+	ctype = nm_setting_connection_get_connection_type (s_con);
+	if (strcmp (ctype, NM_SETTING_WIMAX_SETTING_NAME) != 0)
+		return FALSE;
 
-		ctype = nm_setting_connection_get_connection_type (s_con);
-		if (strcmp (ctype, NM_SETTING_WIMAX_SETTING_NAME) != 0)
-			continue;
+	s_wimax = nm_connection_get_setting_wimax (connection);
+	if (!s_wimax)
+		return FALSE;
 
-		s_wimax = (NMSettingWimax *) nm_connection_get_setting (candidate, NM_TYPE_SETTING_WIMAX);
-		if (!s_wimax)
-			continue;
-
-		/* Check MAC address */
-		hw_str = nm_device_wimax_get_hw_address (NM_DEVICE_WIMAX (device));
-		if (hw_str) {
-			hw_mac = ether_aton (hw_str);
-			mac = nm_setting_wimax_get_mac_address (s_wimax);
-			if (mac && hw_mac && memcmp (mac->data, hw_mac->ether_addr_octet, ETH_ALEN))
-				continue;
-		}
-
-		/* Connection applies to this device */
-		filtered = g_slist_prepend (filtered, candidate);
+	/* Check MAC address */
+	hw_str = nm_device_wimax_get_hw_address (NM_DEVICE_WIMAX (device));
+	if (hw_str) {
+		hw_mac = ether_aton (hw_str);
+		mac = nm_setting_wimax_get_mac_address (s_wimax);
+		if (mac && hw_mac && memcmp (mac->data, hw_mac->ether_addr_octet, ETH_ALEN))
+			return FALSE;
 	}
 
-	return g_slist_reverse (filtered);
+	return TRUE;
 }
 
 /**************************************************************/
@@ -768,7 +759,7 @@ nm_device_wimax_class_init (NMDeviceWimaxClass *wimax_class)
 	object_class->constructor = constructor;
 	object_class->get_property = get_property;
 	object_class->dispose = dispose;
-	device_class->filter_connections = filter_connections;
+	device_class->connection_valid = connection_valid;
 
 	/* properties */
 
