@@ -70,7 +70,6 @@ struct NMPolicy {
 	char *cur_hostname;  /* hostname we want to assign */
 };
 
-#define INVALID_TAG "invalid"
 #define RETRIES_TAG "autoconnect-retries"
 #define RETRIES_DEFAULT	4
 
@@ -239,7 +238,7 @@ _set_hostname (NMPolicy *policy,
 	}
 
 	if (nm_policy_set_system_hostname (policy->cur_hostname, msg))
-		nm_utils_call_dispatcher ("hostname", NULL, NULL, NULL);
+		nm_utils_call_dispatcher ("hostname", NULL, NULL, NULL, NULL, NULL);
 }
 
 static void
@@ -471,7 +470,6 @@ update_ip4_routing_and_dns (NMPolicy *policy, gboolean force_update)
 
 			dns_type = NM_DNS_IP_CONFIG_TYPE_VPN;
 		}
-		g_object_unref (candidate);
 	}
 	g_slist_free (vpns);
 
@@ -541,7 +539,7 @@ update_ip6_routing_and_dns (NMPolicy *policy, gboolean force_update)
 	NMActRequest *best_req = NULL;
 	NMDnsManager *dns_mgr;
 	GSList *devices = NULL, *iter;
-#if NOT_YET
+#if 0
 	GSList *vpns;
 #endif
 	NMIP6Config *ip6_config = NULL;
@@ -557,7 +555,7 @@ update_ip6_routing_and_dns (NMPolicy *policy, gboolean force_update)
 	if (!force_update && (best == policy->default_device6))
 		goto out;
 
-#if NOT_YET
+#if 0
 	/* If a VPN connection is active, it is preferred */
 	vpns = nm_vpn_manager_get_active_connections (policy->vpn_manager);
 	for (iter = vpns; iter; iter = g_slist_next (iter)) {
@@ -596,7 +594,6 @@ update_ip6_routing_and_dns (NMPolicy *policy, gboolean force_update)
 
 			dns_type = NM_DNS_IP_CONFIG_TYPE_VPN;
 		}
-		g_object_unref (candidate);
 	}
 	g_slist_free (vpns);
 #endif
@@ -672,13 +669,15 @@ update_routing_and_dns (NMPolicy *policy, gboolean force_update)
 static void
 set_connection_auto_retries (NMConnection *connection, guint retries)
 {
-	g_object_set_data (G_OBJECT (connection), RETRIES_TAG, GUINT_TO_POINTER (retries));
+	/* add +1 so that the tag still exists if the # retries is 0 */
+	g_object_set_data (G_OBJECT (connection), RETRIES_TAG, GUINT_TO_POINTER (retries + 1));
 }
 
 static guint32
 get_connection_auto_retries (NMConnection *connection)
 {
-	return GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (connection), RETRIES_TAG));
+	/* subtract 1 to handle the +1 from set_connection_auto_retries() */
+	return GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (connection), RETRIES_TAG)) - 1;
 }
 
 typedef struct {
@@ -687,6 +686,16 @@ typedef struct {
 	guint id;
 } ActivateData;
 
+static void
+activate_data_free (ActivateData *data)
+{
+	if (data->id)
+		g_source_remove (data->id);
+	g_object_unref (data->device);
+	memset (data, 0, sizeof (*data));
+	g_free (data);
+}
+
 static gboolean
 auto_activate_device (gpointer user_data)
 {
@@ -699,6 +708,9 @@ auto_activate_device (gpointer user_data)
 	g_assert (data);
 	policy = data->policy;
 
+	data->id = 0;
+	policy->pending_activation_checks = g_slist_remove (policy->pending_activation_checks, data);
+
 	// FIXME: if a device is already activating (or activated) with a connection
 	// but another connection now overrides the current one for that device,
 	// deactivate the device and activate the new connection instead of just
@@ -711,29 +723,15 @@ auto_activate_device (gpointer user_data)
 	/* Remove connections that shouldn't be auto-activated */
 	while (iter) {
 		NMConnection *candidate = NM_CONNECTION (iter->data);
-		gboolean ignore = FALSE;
-
-		/* Ignore connecitons that were tried too many times */
-		if (g_object_get_data (G_OBJECT (candidate), INVALID_TAG)) {
-			guint retries = get_connection_auto_retries (candidate);
-
-			if (retries == 0)
-				ignore = TRUE;
-			else if (retries > 0)
-				set_connection_auto_retries (candidate, retries - 1);
-		} else {
-			/* Set the initial # of retries for auto-connection */
-			set_connection_auto_retries (candidate, RETRIES_DEFAULT);
-		}
-
-		/* Ignore connections that aren't visible to any logged-in users */
-		if (ignore == FALSE) {
-			if (!nm_settings_connection_is_visible (NM_SETTINGS_CONNECTION (candidate)))
-				ignore = TRUE;
-		}
 
+		/* Grab next item before we possibly delete the current item */
 		iter = g_slist_next (iter);
-		if (ignore)
+
+		/* Ignore connections that were tried too many times or are not visible
+		 * to any logged-in users.
+		 */
+		if (   get_connection_auto_retries (candidate) == 0
+		    || nm_settings_connection_is_visible (NM_SETTINGS_CONNECTION (candidate)) == FALSE)
 			connections = g_slist_remove (connections, candidate);
 	}
 
@@ -756,14 +754,37 @@ auto_activate_device (gpointer user_data)
 	g_slist_free (connections);
 
  out:
-	/* Remove this call's handler ID */
-	policy->pending_activation_checks = g_slist_remove (policy->pending_activation_checks, data);
-	g_object_unref (data->device);
-	g_free (data);
-
+	activate_data_free (data);
 	return FALSE;
 }
 
+static ActivateData *
+activate_data_new (NMPolicy *policy, NMDevice *device, guint delay_seconds)
+{
+	ActivateData *data;
+
+	data = g_malloc0 (sizeof (ActivateData));
+	data->policy = policy;
+	data->device = g_object_ref (device);
+	if (delay_seconds > 0)
+		data->id = g_timeout_add_seconds (delay_seconds, auto_activate_device, data);
+	else
+		data->id = g_idle_add (auto_activate_device, data);
+	return data;
+}
+
+static ActivateData *
+find_pending_activation (GSList *list, NMDevice *device)
+{
+	GSList *iter;
+
+	for (iter = list; iter; iter = g_slist_next (iter)) {
+		if (((ActivateData *) iter->data)->device == device)
+			return iter->data;
+	}
+	return NULL;
+}
+
 /*****************************************************************************/
 
 static void
@@ -805,11 +826,11 @@ sleeping_changed (NMManager *manager, GParamSpec *pspec, gpointer user_data)
 	g_object_get (G_OBJECT (manager), NM_MANAGER_SLEEPING, &sleeping, NULL);
 	g_object_get (G_OBJECT (manager), NM_MANAGER_NETWORKING_ENABLED, &enabled, NULL);
 
-	/* Clear the invalid flag on all connections so they'll get retried on wakeup */
+	/* Reset retries on all connections so they'll checked on wakeup */
 	if (sleeping || !enabled) {
 		connections = nm_settings_get_connections (policy->settings);
 		for (iter = connections; iter; iter = g_slist_next (iter))
-			g_object_set_data (G_OBJECT (iter->data), INVALID_TAG, NULL);
+			set_connection_auto_retries (NM_CONNECTION (iter->data), RETRIES_DEFAULT);
 		g_slist_free (connections);
 	}
 }
@@ -818,7 +839,6 @@ static void
 schedule_activate_check (NMPolicy *policy, NMDevice *device, guint delay_seconds)
 {
 	ActivateData *data;
-	GSList *iter;
 	NMDeviceState state;
 
 	if (nm_manager_get_state (policy->manager) == NM_STATE_ASLEEP)
@@ -831,19 +851,11 @@ schedule_activate_check (NMPolicy *policy, NMDevice *device, guint delay_seconds
 	if (!nm_device_autoconnect_allowed (device))
 		return;
 
-	for (iter = policy->pending_activation_checks; iter; iter = g_slist_next (iter)) {
-		/* Only one pending activation check at a time */
-		if (((ActivateData *) iter->data)->device == device)
-			return;
+	/* Schedule an auto-activation if there isn't one already for this device */
+	if (find_pending_activation (policy->pending_activation_checks, device) == NULL) {
+		data = activate_data_new (policy, device, delay_seconds);
+		policy->pending_activation_checks = g_slist_append (policy->pending_activation_checks, data);
 	}
-
-	data = g_malloc0 (sizeof (ActivateData));
-	g_return_if_fail (data != NULL);
-
-	data->policy = policy;
-	data->device = g_object_ref (device);
-	data->id = delay_seconds ? g_timeout_add_seconds (delay_seconds, auto_activate_device, data) : g_idle_add (auto_activate_device, data);
-	policy->pending_activation_checks = g_slist_append (policy->pending_activation_checks, data);
 }
 
 static NMConnection *
@@ -874,7 +886,22 @@ device_state_changed (NMDevice *device,
 		 * it doesn't get automatically chosen over and over and over again.
 		 */
 		if (connection && IS_ACTIVATING_STATE (old_state)) {
-			g_object_set_data (G_OBJECT (connection), INVALID_TAG, GUINT_TO_POINTER (TRUE));
+			guint32 tries = get_connection_auto_retries (connection);
+
+			if (reason == NM_DEVICE_STATE_REASON_NO_SECRETS) {
+				/* If the connection couldn't get the secrets it needed (ex because
+				 * the user canceled, or no secrets exist), there's no point in
+				 * automatically retrying because it's just going to fail anyway.
+				 */
+				set_connection_auto_retries (connection, 0);
+			} else if (tries > 0) {
+				/* Otherwise if it's a random failure, just decrease the number
+				 * of automatic retries so that the connection gets tried again
+				 * if it still has a retry count.
+				 */
+				set_connection_auto_retries (connection, tries - 1);
+			}
+
 			if (get_connection_auto_retries (connection) == 0)
 				nm_log_info (LOGD_DEVICE, "Marking connection '%s' invalid.", nm_connection_get_id (connection));
 			nm_connection_clear_secrets (connection);
@@ -883,10 +910,7 @@ device_state_changed (NMDevice *device,
 		break;
 	case NM_DEVICE_STATE_ACTIVATED:
 		if (connection) {
-			/* Clear the invalid tag on the connection */
-			g_object_set_data (G_OBJECT (connection), INVALID_TAG, NULL);
-
-			/* Reset RETRIES_TAG to number from the setting */
+			/* Reset auto retries back to default since connection was successful */
 			set_connection_auto_retries (connection, RETRIES_DEFAULT);
 
 			/* And clear secrets so they will always be requested from the
@@ -899,7 +923,10 @@ device_state_changed (NMDevice *device,
 		break;
 	case NM_DEVICE_STATE_UNMANAGED:
 	case NM_DEVICE_STATE_UNAVAILABLE:
+		update_routing_and_dns (policy, FALSE);
+		break;
 	case NM_DEVICE_STATE_DISCONNECTED:
+		/* Device is now available for auto-activation */
 		update_routing_and_dns (policy, FALSE);
 		schedule_activate_check (policy, device, 0);
 		break;
@@ -971,21 +998,14 @@ static void
 device_removed (NMManager *manager, NMDevice *device, gpointer user_data)
 {
 	NMPolicy *policy = (NMPolicy *) user_data;
+	ActivateData *tmp;
 	GSList *iter;
 
 	/* Clear any idle callbacks for this device */
-	iter = policy->pending_activation_checks;
-	while (iter) {
-		ActivateData *data = (ActivateData *) iter->data;
-		GSList *next = g_slist_next (iter);
-
-		if (data->device == device) {
-			g_source_remove (data->id);
-			g_object_unref (data->device);
-			g_free (data);
-			policy->pending_activation_checks = g_slist_delete_link (policy->pending_activation_checks, iter);
-		}
-		iter = next;
+	tmp = find_pending_activation (policy->pending_activation_checks, device);
+	if (tmp) {
+		policy->pending_activation_checks = g_slist_remove (policy->pending_activation_checks, tmp);
+		activate_data_free (tmp);
 	}
 
 	/* Clear any signal handlers for this device */
@@ -1020,6 +1040,7 @@ connection_added (NMSettings *settings,
                   NMConnection *connection,
                   gpointer user_data)
 {
+	set_connection_auto_retries (connection, RETRIES_DEFAULT);
 	schedule_activate_all ((NMPolicy *) user_data);
 }
 
@@ -1035,8 +1056,8 @@ connection_updated (NMSettings *settings,
                     NMConnection *connection,
                     gpointer user_data)
 {
-	/* Clear the invalid tag on the connection if it got updated. */
-	g_object_set_data (G_OBJECT (connection), INVALID_TAG, NULL);
+	/* Reset auto retries back to default since connection was updated */
+	set_connection_auto_retries (connection, RETRIES_DEFAULT);
 
 	schedule_activate_all ((NMPolicy *) user_data);
 }
@@ -1173,13 +1194,7 @@ nm_policy_destroy (NMPolicy *policy)
 		policy->lookup = NULL;
 	}
 
-	for (iter = policy->pending_activation_checks; iter; iter = g_slist_next (iter)) {
-		ActivateData *data = (ActivateData *) iter->data;
-
-		g_source_remove (data->id);
-		g_object_unref (data->device);
-		g_free (data);
-	}
+	g_slist_foreach (policy->pending_activation_checks, (GFunc) activate_data_free, NULL);
 	g_slist_free (policy->pending_activation_checks);
 
 	g_signal_handler_disconnect (policy->vpn_manager, policy->vpn_activated_id);
