@@ -19,6 +19,8 @@
  * (C) Copyright 2008 - 2011 Red Hat, Inc.
  */
 
+#include "config.h"
+
 #include <string.h>
 
 #include <NetworkManager.h>
@@ -38,6 +40,8 @@
 #include "nm-marshal.h"
 #include "nm-agent-manager.h"
 
+#define SETTINGS_TIMESTAMPS_FILE  LOCALSTATEDIR"/lib/NetworkManager/timestamps"
+
 static void impl_settings_connection_get_settings (NMSettingsConnection *connection,
                                                    DBusGMethodInvocation *context);
 
@@ -88,6 +92,8 @@ typedef struct {
 
 	NMSessionMonitor *session_monitor;
 	guint session_changed_id;
+
+	guint64 timestamp; /* Up-to-date timestamp of connection use */
 } NMSettingsConnectionPrivate;
 
 /**************************************************************/
@@ -330,6 +336,34 @@ commit_changes (NMSettingsConnection *connection,
 	g_object_unref (connection);
 }
 
+static void
+remove_timestamp_from_db (NMSettingsConnection *connection)
+{
+	GKeyFile *timestamps_file;
+
+	timestamps_file = g_key_file_new ();
+	if (g_key_file_load_from_file (timestamps_file, SETTINGS_TIMESTAMPS_FILE, G_KEY_FILE_KEEP_COMMENTS, NULL)) {
+		const char *connection_uuid;
+		char *data;
+		gsize len;
+		GError *error = NULL;
+
+		connection_uuid = nm_connection_get_uuid (NM_CONNECTION (connection));
+
+		g_key_file_remove_key (timestamps_file, "timestamps", connection_uuid, NULL);
+		data = g_key_file_to_data (timestamps_file, &len, &error);
+		if (data) {
+			g_file_set_contents (SETTINGS_TIMESTAMPS_FILE, data, len, &error);
+			g_free (data);
+		}
+		if (error) {
+			nm_log_warn (LOGD_SETTINGS, "error writing timestamps file '%s': %s", SETTINGS_TIMESTAMPS_FILE, error->message);
+			g_error_free (error);
+		}
+	}
+	g_key_file_free (timestamps_file);
+}
+
 static void
 do_delete (NMSettingsConnection *connection,
            NMSettingsConnectionDeleteFunc callback,
@@ -346,6 +380,9 @@ do_delete (NMSettingsConnection *connection,
 	nm_connection_clear_secrets (for_agents);
 	nm_agent_manager_delete_secrets (priv->agent_mgr, for_agents, FALSE, 0);
 
+	/* Remove timestamp from timestamps database file */
+	remove_timestamp_from_db (connection);
+
 	/* Signal the connection is removed and deleted */
 	g_signal_emit (connection, signals[REMOVED], 0);
 	callback (connection, NULL, user_data);
@@ -449,9 +486,10 @@ static void
 agent_secrets_done_cb (NMAgentManager *manager,
                        guint32 call_id,
                        const char *agent_dbus_owner,
+                       const char *agent_username,
                        gboolean agent_has_modify,
                        const char *setting_name,
-                       guint32 flags,
+                       NMSettingsGetSecretsFlags flags,
                        GHashTable *secrets,
                        GError *error,
                        gpointer user_data,
@@ -464,6 +502,7 @@ agent_secrets_done_cb (NMAgentManager *manager,
 	gpointer callback_data = other_data3;
 	GError *local = NULL;
 	GHashTable *hash;
+	gboolean agent_had_system = FALSE;
 
 	if (error) {
 		nm_log_dbg (LOGD_SETTINGS, "(%s/%s:%u) secrets request error: (%d) %s",
@@ -473,7 +512,7 @@ agent_secrets_done_cb (NMAgentManager *manager,
 		            error->code,
 		            error->message ? error->message : "(unknown)");
 
-		callback (self, call_id, setting_name, error, callback_data);
+		callback (self, call_id, NULL, setting_name, error, callback_data);
 		return;
 	}
 
@@ -481,15 +520,13 @@ agent_secrets_done_cb (NMAgentManager *manager,
 		local = g_error_new (NM_SETTINGS_ERROR, NM_SETTINGS_ERROR_INVALID_SETTING,
 		                     "%s.%d - Connection didn't have requested setting '%s'.",
 		                     __FILE__, __LINE__, setting_name);
-		callback (self, call_id, setting_name, local, callback_data);
+		callback (self, call_id, NULL, setting_name, local, callback_data);
 		g_clear_error (&local);
 		return;
 	}
 
 	g_assert (secrets);
 	if (agent_dbus_owner) {
-		gboolean has_system = FALSE;
-
 		nm_log_dbg (LOGD_SETTINGS, "(%s/%s:%u) secrets returned from agent %s",
 		            nm_connection_get_uuid (NM_CONNECTION (self)),
 		            setting_name,
@@ -502,9 +539,9 @@ agent_secrets_done_cb (NMAgentManager *manager,
 		 * save those system-owned secrets.  If not, discard them and use the
 		 * existing secrets, or fail the connection.
 		 */
-		for_each_secret (NM_CONNECTION (self), secrets, has_system_owned_secrets, &has_system);
-		if (has_system) {
-			if (flags == 0) {  /* ie SECRETS_FLAG_NONE */
+		for_each_secret (NM_CONNECTION (self), secrets, has_system_owned_secrets, &agent_had_system);
+		if (agent_had_system) {
+			if (flags == NM_SETTINGS_GET_SECRETS_FLAG_NONE) {
 				/* No user interaction was allowed when requesting secrets; the
 				 * agent is being bad.  Remove system-owned secrets.
 				 */
@@ -542,7 +579,7 @@ agent_secrets_done_cb (NMAgentManager *manager,
 	/* If no user interaction was allowed, make sure that no "unsaved" secrets
 	 * came back.  Unsaved secrets by definition require user interaction.
 	 */
-	if (flags == 0)  /* ie SECRETS_FLAG_NONE */
+	if (flags == NM_SETTINGS_GET_SECRETS_FLAG_NONE)
 		for_each_secret (NM_CONNECTION (self), secrets, clear_unsaved_secrets, NULL);
 
 	/* Update the connection with our existing secrets from backing storage */
@@ -560,12 +597,24 @@ agent_secrets_done_cb (NMAgentManager *manager,
 			 */
 			update_secrets_cache (self);
 
-			nm_log_dbg (LOGD_SETTINGS, "(%s/%s:%u) saving new secrets to backing storage",
-			            nm_connection_get_uuid (NM_CONNECTION (self)),
-			            setting_name,
-			            call_id);
+			/* Only save secrets to backing storage if the agent returned any
+			 * new system secrets.  If it didn't, then the secrets are agent-
+			 * owned and there's no point to writing out the connection when
+			 * nothing has changed, since agent-owned secrets don't get saved here.
+			 */
+			if (agent_had_system) {
+				nm_log_dbg (LOGD_SETTINGS, "(%s/%s:%u) saving new secrets to backing storage",
+						    nm_connection_get_uuid (NM_CONNECTION (self)),
+						    setting_name,
+						    call_id);
 
-			nm_settings_connection_commit_changes (self, new_secrets_commit_cb, NULL);
+				nm_settings_connection_commit_changes (self, new_secrets_commit_cb, NULL);
+			} else {
+				nm_log_dbg (LOGD_SETTINGS, "(%s/%s:%u) new agent secrets processed",
+						    nm_connection_get_uuid (NM_CONNECTION (self)),
+						    setting_name,
+						    call_id);
+			}
 		} else {
 			nm_log_dbg (LOGD_SETTINGS, "(%s/%s:%u) failed to update with agent secrets: (%d) %s",
 			            nm_connection_get_uuid (NM_CONNECTION (self)),
@@ -583,7 +632,7 @@ agent_secrets_done_cb (NMAgentManager *manager,
 		            (local && local->message) ? local->message : "(unknown)");
 	}
 
-	callback (self, call_id, setting_name, local, callback_data);
+	callback (self, call_id, agent_username, setting_name, local, callback_data);
 	g_clear_error (&local);
 	if (hash)
 		g_hash_table_destroy (hash);
@@ -612,14 +661,13 @@ nm_settings_connection_get_secrets (NMSettingsConnection *self,
                                     gboolean filter_by_uid,
                                     gulong uid,
                                     const char *setting_name,
-                                    guint32 flags,
+                                    NMSettingsGetSecretsFlags flags,
                                     const char *hint,
                                     NMSettingsConnectionSecretsFunc callback,
                                     gpointer callback_data,
                                     GError **error)
 {
 	NMSettingsConnectionPrivate *priv = NM_SETTINGS_CONNECTION_GET_PRIVATE (self);
-	NMSettingConnection *s_con;
 	GHashTable *existing_secrets;
 	guint32 call_id = 0;
 
@@ -659,9 +707,8 @@ nm_settings_connection_get_secrets (NMSettingsConnection *self,
 	if (existing_secrets)
 		g_hash_table_unref (existing_secrets);
 
-	s_con = (NMSettingConnection *) nm_connection_get_setting (NM_CONNECTION (self), NM_TYPE_SETTING_CONNECTION);
 	nm_log_dbg (LOGD_SETTINGS, "(%s/%s:%u) secrets requested flags 0x%X hint '%s'",
-	            nm_setting_connection_get_uuid (s_con),
+	            nm_connection_get_uuid (NM_CONNECTION (self)),
 	            setting_name,
 	            call_id,
 	            flags,
@@ -675,11 +722,9 @@ nm_settings_connection_cancel_secrets (NMSettingsConnection *self,
                                        guint32 call_id)
 {
 	NMSettingsConnectionPrivate *priv = NM_SETTINGS_CONNECTION_GET_PRIVATE (self);
-	NMSettingConnection *s_con;
 
-	s_con = (NMSettingConnection *) nm_connection_get_setting (NM_CONNECTION (self), NM_TYPE_SETTING_CONNECTION);
 	nm_log_dbg (LOGD_SETTINGS, "(%s:%u) secrets canceled",
-	            nm_setting_connection_get_uuid (s_con),
+	            nm_connection_get_uuid (NM_CONNECTION (self)),
 	            call_id);
 
 	priv->reqs = g_slist_remove (priv->reqs, GUINT_TO_POINTER (call_id));
@@ -865,15 +910,35 @@ get_settings_auth_cb (NMSettingsConnection *self,
 		dbus_g_method_return_error (context, error);
 	else {
 		GHashTable *settings;
+	 	NMConnection *dupl_con;
+		NMSettingConnection *s_con;
+		guint64 timestamp;
+
+	 	dupl_con = nm_connection_duplicate (NM_CONNECTION (self));
+ 		g_assert (dupl_con);
+
+		/* Timestamp is not updated in connection's 'timestamp' property,
+		 * because it would force updating the connection and in turn
+		 * writing to /etc periodically, which we want to avoid. Rather real
+		 * timestamps are kept track of in a private variable. So, substitute
+		 * timestamp property with the real one here before returning the settings.
+		 */
+		timestamp = nm_settings_connection_get_timestamp (self);
+		if (timestamp) {
+			s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (NM_CONNECTION (dupl_con), NM_TYPE_SETTING_CONNECTION));
+			g_assert (s_con);
+			g_object_set (s_con, NM_SETTING_CONNECTION_TIMESTAMP, timestamp, NULL);
+		}
 
 		/* Secrets should *never* be returned by the GetSettings method, they
 		 * get returned by the GetSecrets method which can be better
 		 * protected against leakage of secrets to unprivileged callers.
 		 */
-		settings = nm_connection_to_hash (NM_CONNECTION (self), NM_SETTING_HASH_FLAG_NO_SECRETS);
+		settings = nm_connection_to_hash (NM_CONNECTION (dupl_con), NM_SETTING_HASH_FLAG_NO_SECRETS);
 		g_assert (settings);
 		dbus_g_method_return (context, settings);
 		g_hash_table_destroy (settings);
+ 		g_object_unref (dupl_con);
 	}
 }
 
@@ -1103,6 +1168,7 @@ impl_settings_connection_delete (NMSettingsConnection *self,
 static void
 dbus_get_agent_secrets_cb (NMSettingsConnection *self,
                            guint32 call_id,
+                           const char *agent_username,
                            const char *setting_name,
                            GError *error,
                            gpointer user_data)
@@ -1149,7 +1215,7 @@ dbus_secrets_auth_cb (NMSettingsConnection *self,
 			                                          TRUE,
 			                                          sender_uid,
 			                                          setting_name,
-			                                          0, /* GET_SECRETS_FLAG_NONE */
+			                                          NM_SETTINGS_GET_SECRETS_FLAG_NONE,
 			                                          NULL,
 			                                          dbus_get_agent_secrets_cb,
 			                                          context,
@@ -1194,6 +1260,106 @@ nm_settings_connection_signal_remove (NMSettingsConnection *self)
 	g_signal_emit_by_name (self, "unregister");
 }
 
+/**
+ * nm_settings_connection_get_timestamp:
+ * @connection: the #NMSettingsConnection
+ *
+ * Returns current connection's timestamp.
+ *
+ * Returns: timestamp of the last connection use (0 when it's not used)
+ **/
+guint64
+nm_settings_connection_get_timestamp (NMSettingsConnection *connection)
+{
+	g_return_val_if_fail (NM_SETTINGS_CONNECTION (connection), 0);
+
+	return NM_SETTINGS_CONNECTION_GET_PRIVATE (connection)->timestamp;
+}
+
+/**
+ * nm_settings_connection_update_timestamp:
+ * @connection: the #NMSettingsConnection
+ * @timestamp: timestamp to set into the connection and to store into
+ * the timestamps database
+ *
+ * Updates the connection and timestamps database with the provided timestamp.
+ **/
+void
+nm_settings_connection_update_timestamp (NMSettingsConnection *connection, guint64 timestamp)
+{
+	NMSettingsConnectionPrivate *priv = NM_SETTINGS_CONNECTION_GET_PRIVATE (connection);
+	const char *connection_uuid;
+	GKeyFile *timestamps_file;
+	char *data, *tmp;
+	gsize len;
+	GError *error = NULL;
+
+	/* Update timestamp in private storage */
+	priv->timestamp = timestamp;
+
+	/* Save timestamp to timestamps database file */
+	timestamps_file = g_key_file_new ();
+	if (!g_key_file_load_from_file (timestamps_file, SETTINGS_TIMESTAMPS_FILE, G_KEY_FILE_KEEP_COMMENTS, &error)) {
+		if (!(error->domain == G_FILE_ERROR && error->code == G_FILE_ERROR_NOENT))
+			nm_log_warn (LOGD_SETTINGS, "error parsing timestamps file '%s': %s", SETTINGS_TIMESTAMPS_FILE, error->message);
+		g_clear_error (&error);
+	}
+
+	connection_uuid = nm_connection_get_uuid (NM_CONNECTION (connection));
+	tmp = g_strdup_printf ("%" G_GUINT64_FORMAT, timestamp);
+	g_key_file_set_value (timestamps_file, "timestamps", connection_uuid, tmp);
+	g_free (tmp);
+ 
+	data = g_key_file_to_data (timestamps_file, &len, &error);
+	if (data) {
+		g_file_set_contents (SETTINGS_TIMESTAMPS_FILE, data, len, &error);
+		g_free (data);
+	}
+	if (error) {
+		nm_log_warn (LOGD_SETTINGS, "error saving timestamp to file '%s': %s", SETTINGS_TIMESTAMPS_FILE, error->message);
+		g_error_free (error);
+	}
+	g_key_file_free (timestamps_file);
+}
+
+/**
+ * nm_settings_connection_read_and_fill_timestamp:
+ * @connection: the #NMSettingsConnection
+ *
+ * Retrieves timestamp of the connection's last usage from database file and
+ * stores it into the connection private data.
+ **/
+void
+nm_settings_connection_read_and_fill_timestamp (NMSettingsConnection *connection)
+{
+	NMSettingsConnectionPrivate *priv = NM_SETTINGS_CONNECTION_GET_PRIVATE (connection);
+	const char *connection_uuid;
+	guint64 timestamp = 0;
+	GKeyFile *timestamps_file;
+	GError *err = NULL;
+	char *tmp_str;
+
+	/* Get timestamp from database file */
+	timestamps_file = g_key_file_new ();
+	g_key_file_load_from_file (timestamps_file, SETTINGS_TIMESTAMPS_FILE, G_KEY_FILE_KEEP_COMMENTS, NULL);
+	connection_uuid = nm_connection_get_uuid (NM_CONNECTION (connection));
+	tmp_str = g_key_file_get_value (timestamps_file, "timestamps", connection_uuid, &err);
+	if (tmp_str) {
+		timestamp = g_ascii_strtoull (tmp_str, NULL, 10);
+		g_free (tmp_str);
+	}
+
+	/* Update connection's timestamp */
+	if (!err)
+		priv->timestamp = timestamp;
+	else {
+		nm_log_dbg (LOGD_SETTINGS, "failed to read connection timestamp for '%s': (%d) %s",
+		            connection_uuid, err->code, err->message);
+		g_clear_error (&err);
+	}
+	g_key_file_free (timestamps_file);
+}
+
 /**************************************************************/
 
 static void
