@@ -695,6 +695,14 @@ nm_supplicant_config_add_setting_wireless_security (NMSupplicantConfig *self,
 			if (!nm_supplicant_config_add_setting_8021x (self, setting_8021x, connection_uid, FALSE))
 				return FALSE;
 		}
+
+		if (!strcmp (key_mgmt, "wpa-eap")) {
+			/* If using WPA Enterprise, enable optimized background scanning
+			 * to ensure roaming within an ESS works well.
+			 */
+			if (!nm_supplicant_config_add_option (self, "bgscan", "simple:30:-45:300", -1, FALSE))
+				nm_log_warn (LOGD_SUPPLICANT, "Error enabling background scanning for ESS roaming");
+		}
 	}
 
 	return TRUE;
@@ -731,6 +739,7 @@ nm_supplicant_config_add_setting_8021x (NMSupplicantConfig *self,
 		/* Wired 802.1x must always use eapol_flags=0 */
 		if (!add_string_val (self, "0", "eapol_flags", FALSE, FALSE))
 			return FALSE;
+		nm_supplicant_config_set_ap_scan (self, 0);
 	}
 
 	ADD_STRING_LIST_VAL (setting, 802_1x, eap_method, eap_methods, "eap", TRUE, FALSE);
